"""Units tests for User rest api
"""
from django.test import TestCase

from core_main_app.rest.user import views as user_rest_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGetAllUserList(TestCase):
    def setUp(self):
        super().setUp()
        self.data = None

    def test_get_all_returns_list(self):
        # Arrange
        user = create_mock_user("0", is_staff=True)

        # Act
        response = RequestMock.do_request_get(
            user_rest_views.UserListCreateView.as_view(), user, self.data
        )

        # Assert
        excepted_result = []
        self.assertEqual(response.data, excepted_result)



"""Unit tests for template rest api
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template.models import Template

from core_main_app.rest.template import views as template_rest_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
import core_main_app.components.template.api as template_api


class TestTemplateDownload(SimpleTestCase):
    def setUp(self):
        super(TestTemplateDownload, self).setUp()

    @patch.object(template_api, "get")
    def test_get_returns_http_404_when_data_not_found(self, mock_template_api_get):
        # Arrange
        mock_user = create_mock_user("1")
        mock_template_api_get.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(), mock_user, param={"pk": "1"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @patch.object(template_api, "get")
    def test_get_returns_http_200_when_data_found(self, mock_template_api_get):
        # Arrange
        mock_user = create_mock_user("1")
        mock_template = _get_template()
        mock_template_api_get.return_value = mock_template

        # Mock
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(), mock_user, param={"pk": "1"}
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(template_api, "get")
    def test_get_returns_http_400_when_data_not_well_formatted(
        self, mock_template_api_get
    ):
        # Arrange
        mock_user = create_mock_user("1")
        mock_template = Template()
        mock_template_api_get.return_value = mock_template

        # Mock
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            mock_user,
            param={"pk": "1"},
            data={"pretty_print": "true"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @patch.object(template_api, "get")
    def test_get_without_pretty_print_returns_data(self, mock_template_api_get):
        # Arrange
        mock_user = create_mock_user("1")
        mock_template = _get_template()
        mock_template_api_get.return_value = mock_template
        expected_value = b'<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema"><xs:element name="tag"></xs:element></xs:schema>'
        # Mock
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            mock_user,
            param={"pk": "1"},
            data={"pretty_print": "false"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.content, expected_value)

    @patch.object(template_api, "get")
    def test_get_with_pretty_print_returns_formatted_data(self, mock_template_api_get):
        # Arrange
        mock_user = create_mock_user("1")
        mock_template = _get_template()
        mock_template_api_get.return_value = mock_template
        expected_value = b'<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">\n  <xs:element name="tag"/>\n</xs:schema>\n'

        # Mock
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            mock_user,
            param={"pk": "1"},
            data={"pretty_print": "true"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.content, expected_value)


def _get_template():
    template = Template()
    template.id_field = 1
    template.filename = "test"
    xsd = (
        '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
        '<xs:element name="tag"></xs:element></xs:schema>'
    )
    template.content = xsd
    return template



""" Utils for djangosaml2 package
"""
import os

import saml2
import saml2.saml


def load_attribute_map_from_env():
    """Load attribute map from environment (https://pysaml2.readthedocs.io/en/latest/howto/config.html#attribute-map-dir)

    Returns:

    """
    return {
        "identifier": os.environ["SAML_ATTRIBUTES_MAP_IDENTIFIER"],
        "fro": {
            os.environ["SAML_ATTRIBUTES_MAP_UID"]: os.environ[
                "SAML_ATTRIBUTES_MAP_UID_FIELD"
            ],
            os.environ["SAML_ATTRIBUTES_MAP_EMAIL"]: os.environ[
                "SAML_ATTRIBUTES_MAP_EMAIL_FIELD"
            ],
            os.environ["SAML_ATTRIBUTES_MAP_SN"]: os.environ[
                "SAML_ATTRIBUTES_MAP_SN_FIELD"
            ],
            os.environ["SAML_ATTRIBUTES_MAP_CN"]: os.environ[
                "SAML_ATTRIBUTES_MAP_CN_FIELD"
            ],
        },
        "to": {
            os.environ["SAML_ATTRIBUTES_MAP_UID_FIELD"]: os.environ[
                "SAML_ATTRIBUTES_MAP_UID"
            ],
            os.environ["SAML_ATTRIBUTES_MAP_EMAIL_FIELD"]: os.environ[
                "SAML_ATTRIBUTES_MAP_EMAIL"
            ],
            os.environ["SAML_ATTRIBUTES_MAP_SN_FIELD"]: os.environ[
                "SAML_ATTRIBUTES_MAP_SN"
            ],
            os.environ["SAML_ATTRIBUTES_MAP_CN_FIELD"]: os.environ[
                "SAML_ATTRIBUTES_MAP_CN"
            ],
        },
    }


def load_django_attribute_map_from_env():
    """Load attribute mapping for Django from environment (https://djangosaml2.readthedocs.io/contents/setup.html#users-attributes-and-account-linking)

    Returns:

    """
    return {
        os.environ["SAML_ATTRIBUTES_MAP_UID_FIELD"]: ("username",),
        os.environ["SAML_ATTRIBUTES_MAP_EMAIL_FIELD"]: ("email",),
        os.environ["SAML_ATTRIBUTES_MAP_CN_FIELD"]: ("first_name",),
        os.environ["SAML_ATTRIBUTES_MAP_SN_FIELD"]: ("last_name",),
    }


def load_saml_config_from_env(server_uri, base_dir):
    """Load SAML configuration from environment (https://pysaml2.readthedocs.io/en/latest/howto/config.html#configuration-of-pysaml2-entities)

    Args:
        server_uri:
        base_dir:

    Returns:

    """

    contact_person = load_contact_person_from_env()
    organization = load_organization_from_env()
    return {
        # full path to the xmlsec1 binary programm
        "xmlsec_binary": os.getenv("SAML_XMLSEC_BIN_PATH", "/usr/bin/xmlsec1"),
        # your entity id, usually your subdomain plus the url to the metadata view
        "entityid": f"{server_uri}/saml2/metadata/",
        # directory with attribute mapping
        "attribute_map_dir": os.path.join(
            base_dir, os.getenv("SAML_ATTRIBUTE_MAP_DIR", "attr-maps")
        ),
        # this block states what services we provide
        "service": {
            # we are just a lonely SP
            "sp": {
                "want_response_signed": os.getenv(
                    "SAML_WANT_RESPONSE_SIGNED", "False"
                ).lower()
                == "true",
                "want_assertions_signed": os.getenv(
                    "SAML_WANT_ASSERTIONS_SIGNED", "False"
                ).lower()
                == "true",
                "name": os.getenv("SERVER_NAME", "Curator"),
                "name_id_format": saml2.saml.NAMEID_FORMAT_TRANSIENT,
                # For Okta add signed logout requets. Enable this:
                "logout_requests_signed": os.getenv(
                    "SAML_LOGOUT_REQUESTS_SIGNED", "True"
                ).lower()
                == "true",
                "logout_responses_signed": os.getenv(
                    "SAML_LOGOUT_RESPONSES_SIGNED", "False"
                ).lower()
                == "true",
                "signing_algorithm": os.getenv(
                    "SAML_SIGNING_ALGORITHM",
                    "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512",
                ),
                "digest_algorithm": os.getenv(
                    "SAML_DIGEST_ALGORITHM", "http://www.w3.org/2001/04/xmlenc#sha512"
                ),
                "endpoints": {
                    # url and binding to the assertion consumer service view
                    # do not change the binding or service name
                    "assertion_consumer_service": [
                        (f"{server_uri}/saml2/acs/", saml2.BINDING_HTTP_POST),
                    ],
                    # url and binding to the single logout service view
                    # do not change the binding or service name
                    "single_logout_service": [
                        # Disable next two lines for HTTP_REDIRECT for IDP's that only support HTTP_POST. Ex. Okta:
                        (f"{server_uri}/saml2/ls/", saml2.BINDING_HTTP_REDIRECT),
                        (f"{server_uri}/saml2/ls/post/", saml2.BINDING_HTTP_POST),
                    ],
                },
                # Mandates that the identity provider MUST authenticate the
                # presenter directly rather than rely on a previous security context.
                "force_authn": False,
                # Enable AllowCreate in NameIDPolicy.
                "name_id_format_allow_create": False,
                # attributes that this project need to identify a user
                "required_attributes": ["givenName", "sn", "mail"],
            },
        },
        # where the remote metadata is stored, local, remote or mdq server.
        # One metadata store or many ...
        "metadata": {
            "remote": [
                {
                    "url": os.environ["SAML_METADATA_REMOTE"],
                    "cert": os.getenv("SAML_METADATA_REMOTE_CERT"),
                },
            ]
            if "SAML_METADATA_REMOTE" in os.environ
            else [],
            "local": [os.environ["SAML_METADATA_LOCAL"]]
            if "SAML_METADATA_LOCAL" in os.environ
            else [],
        },
        # Signing
        "key_file": os.getenv("SAML_KEY_FILE", None),  # private part
        "cert_file": os.getenv("SAML_CERT_FILE", None),  # public part
        # Encryption
        "encryption_keypairs": [
            {
                "key_file": os.getenv("SAML_KEY_FILE", None),  # private part
                "cert_file": os.getenv("SAML_CERT_FILE", None),  # public part
            }
        ],
        "contact_person": contact_person,
        "organization": organization,
    }


def load_contact_person_from_env():
    """Load PySAML2 contact person configurations from environment (https://pysaml2.readthedocs.io/en/latest/howto/config.html#contact-person)

    Examples:
        CONTACT_PERSON_1=Firstname1,Lastname1,Example Co.,contact1@example.com,technical
        CONTACT_PERSON_2=Firstname2,Lastname2,Example Co.,contact2@example.com,administrative

    Returns:

    """
    contact_person_list = list()
    contact_person_count = 0
    while 1:
        contact_person = os.getenv(f"CONTACT_PERSON_{str(contact_person_count+1)}")
        if contact_person:
            contact_person_info = contact_person.split(",")
            contact_person_list.append(
                {
                    "given_name": contact_person_info[0],
                    "sur_name": contact_person_info[1],
                    "company": contact_person_info[2],
                    "email_address": contact_person_info[3],
                    "contact_type": contact_person_info[4],
                }
            )
            contact_person_count += 1
        else:
            break

    return contact_person_list


def load_organization_from_env():
    """Load PySAML2 organization configurations from environment (https://pysaml2.readthedocs.io/en/latest/howto/config.html#organization)

    Examples:
        ORGANIZATION_NAME_1=Example Company,en
        ORGANIZATION_NAME_2=Exempel AB,se
        ORGANIZATION_DISPLAY_NAME_1=Exempel AB,se
        ORGANIZATION_URL_1=http://example.com,en
        ORGANIZATION_URL_2=http://exemple.se,se

    Returns:

    """
    organization_dict = dict()

    for organization_config in ["name", "display_name", "url"]:
        organization_dict[organization_config] = list()
        organization_config_count = 0
        while 1:
            organization_config_value = os.getenv(
                f"ORGANIZATION_{organization_config.upper()}_{str(organization_config_count+1)}"
            )
            if organization_config_value:
                organization_dict[organization_config].append(
                    tuple(organization_config_value.split(","))
                )
                organization_config_count += 1
            else:
                break

    return organization_dict



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-09-26 14:40
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Main",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "core_main_app",
                "permissions": (("publish_data", "Can publish data"),),
                "default_permissions": (),
            },
        ),
    ]



"""Mongo query builder tools
"""
import copy
import re

from mongoengine.queryset.visitor import Q

from core_main_app.commons.exceptions import QueryError


def _compile_regex(query):
    """Compile all regular expressions in the query

    Args:
        query:

    Returns:

    """
    for key, value in query.items():
        if key == "$and" or key == "$or":
            for sub_value in value:
                _compile_regex(sub_value)
        elif isinstance(value, str) or isinstance(value, str):
            if len(value) >= 2 and value[0] == "/" and value[-1] == "/":
                query[key] = re.compile(value[1:-1])
        elif isinstance(value, dict):
            _compile_regex(value)


def _add_sub_document_root(query, sub_document_root):
    """Adds a sub document root to each criteria

    Returns:

    """
    for key in list(query.keys()):
        if key == "$and" or key == "$or":
            for value in query[key]:
                _add_sub_document_root(value, sub_document_root)
        elif not key.startswith("$"):
            query["{}.{}".format(sub_document_root, key)] = query.pop(key)


def prepare_query(query_dict, regex=True, sub_document_root=None):
    """Prepares the query to before executing it

    Args:
        query_dict:
        regex:
        sub_document_root:

    Returns:

    """
    # get a copy of the query
    query = copy.deepcopy(query_dict)

    if regex:
        # compile the regular expressions
        _compile_regex(query)

    if sub_document_root is not None:
        # add a sub document root
        _add_sub_document_root(query, sub_document_root)

    return query


def sanitize(query_dict):
    """Sanitize query

    Returns:

    """
    # create a query object
    q_list = Q()

    # create $where operator
    if "$where" in str(query_dict):
        raise QueryError("Unsupported operator found")

    # iterate through query dict key/value pairs
    for key, value in query_dict.items():
        # if the key is not an operator
        if not key.startswith("$"):
            if "$" in key:
                raise QueryError("Unsupported $ operator found")
            # replace dots by double underscores (django notation)
            key = key.replace(".", "__") if "." in key else key
            # initialize operator
            operator = "exact"
            # check if not operator
            if isinstance(value, dict) and "$not" in value:
                # add not to key
                key += "__not"
                # move value to document in $not
                value = value["$not"]
            # if value is a regex
            if isinstance(value, re.Pattern):
                # set regex operator
                operator = "regex"
                # set value with regex pattern
                value = value.pattern
            # if value is None
            elif value is None:
                # mongo: __exact = None / django: __exact = None
                value = None
            # if the value is a dict
            elif isinstance(value, dict):
                # check if ne operator (not equal)
                if "$ne" in value:
                    # set ne operator
                    operator = "ne"
                    # set value
                    value = value["$ne"]
                # check if eq operator (equal to)
                elif "$eq" in value:
                    # set value
                    value = value["$eq"]
                # check if lt operator (less than)
                elif "$lt" in value:
                    # set lt operator
                    operator = "lt"
                    # set value
                    value = sanitize_number(value["$lt"])
                # check if lt operator (less than or equal)
                elif "$lte" in value:
                    # set lte operator
                    operator = "lte"
                    # set value
                    value = sanitize_number(value["$lte"])
                # check if gt operator (greater than)
                elif "$gt" in value:
                    # set gt operator
                    operator = "gt"
                    # set value
                    value = sanitize_number(value["$gt"])
                # check if gte operator (greater than or equal)
                elif "$gte" in value:
                    # set gte operator
                    operator = "gte"
                    # set value
                    value = sanitize_number(value["$gte"])
                # check if in operator (included in list)
                elif "$in" in value:
                    # set in operator
                    operator = "in"
                    # set value
                    value = value["$in"]
                # check if regex operator
                elif "$regex" in value:
                    # set regex operator
                    operator = "regex"
                    # set the value
                    value = value["$regex"]
                # check if exists operator
                elif "$exists" in value:
                    # skip case where set to False for now (i.e. can not look for documents where path is absent)
                    if not value["$exists"]:
                        raise QueryError(
                            'Unsupported operator found: {"$exists": False}'
                        )
                    # set exists operator
                    operator = "exists"
                    value = True
                else:
                    # If an operator not listed above is found, an exception is raised
                    raise QueryError(f"Unsupported operator found: {value}")
            # build query
            query = Q(**{f"{key}__{operator}": sanitize_value(value)})

            # add AND query filter from string key and value
            q_list &= query
        else:
            # if operator and
            if key == "$and":
                # iterate though sub dict
                for sub_value in value:
                    # add AND filters to the query
                    q_list &= sanitize(sub_value)
            # if operator or
            elif key == "$or":
                # iterate through sub dict
                for sub_value in value:
                    # add OR filters to the query
                    q_list |= sanitize(sub_value)
            # if operators text and search found
            elif key == "$text" and "$search" in value:
                # get text query
                text_query = query_dict["$text"]["$search"]
                # check text query is a string
                if not isinstance(text_query, str):
                    raise QueryError(f"Unsupported value found in: {key}")
                # strip white spaces
                text_query = text_query.strip()
                # sanitize string
                sanitize_value(text_query)
                # if data stored in MongoDB
                q_list &= Q(__raw__={"$text": {"$search": text_query}})
            else:
                # raise an error if another operator was found
                raise QueryError(f"Unsupported operator found: {key}")

    return q_list


def sanitize_number(value):
    """Sanitize number


    Args:
        value:

    Returns:

    """
    # check type of value
    if not (isinstance(value, (int, float))):
        raise QueryError("Unsupported value: expected a number.")
    return value


def sanitize_value(value):
    """Sanitize value

    Args:
        value:

    Returns:

    """
    # check if operator in value
    if "$" in str(value):
        raise QueryError("Unsupported $ operator found.")
    # if value is a list
    if isinstance(value, list):
        # check for $ sign in the list
        for item in value:
            sanitize_value(item)
    return value



""" Authentication tests for User REST API
"""
from django.test import TestCase
from rest_framework import status

from core_main_app.rest.user import views as user_rest_views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import (
    MongoIntegrationTransactionTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.user.fixtures.fixtures import UserFixtures

user_fixture = UserFixtures()


class TestUserGetPermissions(MongoIntegrationTransactionTestCase):
    def setUp(self):
        super().setUp()

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            user_rest_views.UserRetrieveUpdateView.as_view(), None, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            user_rest_views.UserRetrieveUpdateView.as_view(), mock_user, param={"pk": 0}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_200(self):
        mock_get_user = user_fixture.create_user("mock_user")

        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            user_rest_views.UserRetrieveUpdateView.as_view(),
            mock_user,
            param={"pk": mock_get_user.id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserListGetPermissions(TestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            user_rest_views.UserListCreateView.as_view(), None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        mock_user = create_mock_user("1")
        response = RequestMock.do_request_get(
            user_rest_views.UserListCreateView.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_returns_http_200(self):
        mock_user = create_mock_user("1", is_staff=True)
        response = RequestMock.do_request_get(
            user_rest_views.UserListCreateView.as_view(), mock_user
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



""" File utils
"""
import base64
import re
from io import BytesIO
from mimetypes import guess_type

from django.http.response import HttpResponse

from core_main_app.commons.exceptions import CoreError


def get_file_http_response(file_content, file_name, content_type=None, extension=""):
    """Return http response with file to download

    Args:
        file_content:
        file_name:
        content_type:
        extension:

    Returns:

    """
    try:
        # set file content
        try:
            _file = BytesIO(file_content.encode("utf-8"))
        except Exception:
            _file = BytesIO(file_content)

        # guess file content type if not set
        if content_type is None:
            # The function 'guess_type' returns a tuple (type, encoding). The
            # HttpResponse only needs the type of the file sent, otherwise it
            # cannot detect the proper content type of the file.
            #
            # See https://docs.python.org/3/library/mimetypes.html#mimetypes.guess_type
            content_type = guess_type(file_name)[0]
        # set file in http response
        response = HttpResponse(_file, content_type=content_type)
        # set filename extension
        if not file_name.endswith(extension):
            if not extension.startswith("."):
                extension = "." + extension
            file_name += extension
        # set content disposition in response
        response["Content-Disposition"] = "attachment; filename=" + file_name
        # return response
        return response
    except Exception:
        raise CoreError("An unexpected error occurred.")


def read_file_content(file_path):
    """Read the content of a file

    Args:
        file_path:

    Returns:

    """
    with open(file_path) as _file:
        file_content = _file.read()
        return file_content


def get_filename_from_response(response):
    """Get filename from HTTP response

    Args:
        response: HTTP response

    Returns:

    """
    content_disposition = response.headers.get("content-disposition")
    if not content_disposition:
        return None
    file_name = re.findall("filename=(.+)", content_disposition)
    if len(file_name) == 0:
        return None
    return file_name[0]


def get_base_64_content_from_response(response):
    """Get ascii content from HTTP response

    Args:
        response: HTTP response

    Returns:

    """
    try:
        b64_content = base64.b64encode(response.content)
        return b64_content.decode("ascii")
    except Exception:
        raise CoreError("An error occurred while decoding the response.")



"""
    Common ajax
"""

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.http.response import JsonResponse
from django.urls import reverse_lazy
from django.utils.decorators import method_decorator
from django_mongoengine.views import UpdateView, DeleteView, CreateView

from core_main_app.commons import exceptions
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.components.template_version_manager.models import (
    TemplateVersionManager,
)
from core_main_app.views.admin.forms import EditTemplateForm


class AddObjectModalView(CreateView):
    """Common AddObjectModalView.
    Should be used with add.html and add.js.
    """

    template_name = "core_main_app/common/commons/form.html"
    form_class = None
    document = None
    success_url = None
    success_message = None

    def form_invalid(self, form):
        # Get initial response
        response = super(AddObjectModalView, self).form_invalid(form)
        data = {"is_valid": False, "responseText": response.rendered_content}
        return JsonResponse(data)

    def form_valid(self, form):
        # This method is called when valid form data has been POSTed.
        # Call private save method
        # Populate self.object without committing to the database
        self.object = form.save(commit=False)
        self._save(form)
        if form.is_valid():
            data = {
                "is_valid": True,
                "url": self.get_success_url(),
            }

            if self.success_message:
                messages.success(self.request, self.success_message)

            return JsonResponse(data)
        else:
            return self.form_invalid(form)

    def _save(self, form):
        # Save treatment.
        super(AddObjectModalView, self).form_valid(form)

    @staticmethod
    def get_modal_html_path():
        return "core_main_app/common/modals/add_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        return {"path": "core_main_app/common/js/modals/add.js", "is_raw": False}


class EditObjectModalView(UpdateView):
    """Common EditObjectModalView.
    Should be used with edit_page_modal.html and edit.js.
    """

    template_name = "core_main_app/common/commons/form.html"
    form_class = None
    document = None
    success_url = None
    success_message = None

    def form_invalid(self, form):
        # Get initial response
        response = super(EditObjectModalView, self).form_invalid(form)
        data = {"is_valid": False, "responseText": response.rendered_content}
        return JsonResponse(data)

    def form_valid(self, form):
        # This method is called when valid form data has been POSTed.
        # Call private save method
        self._save(form)
        if form.is_valid():
            data = {
                "is_valid": True,
                "url": self.get_success_url(),
            }

            if self.success_message:
                messages.success(self.request, self.success_message)

            return JsonResponse(data)
        else:
            return self.form_invalid(form)

    def _save(self, form):
        # Save treatment.
        super(EditObjectModalView, self).form_valid(form)

    @staticmethod
    def get_modal_html_path():
        return "core_main_app/common/modals/edit_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        return {"path": "core_main_app/common/js/modals/edit.js", "is_raw": False}


class DeleteObjectModalView(DeleteView):
    """Common DeleteObjectModalView.
    Should be used with delete_page_modal.html and delete.js.
    """

    template_name = "core_main_app/common/commons/form_delete.html"
    document = None
    field_for_name = None
    success_url = None
    success_message = None

    def delete(self, request, *args, **kwargs):
        """
        Delete method.
        """
        try:
            self.object = self.get_object()
            self._delete(request, *args, **kwargs)

            if self.success_message:
                messages.success(self.request, self.success_message)
        except Exception as e:
            messages.error(self.request, str(e))

        data = {"url": self.get_success_url()}
        return JsonResponse(data)

    def _delete(self, request, *args, **kwargs):
        """
        Delete treatment.
        """
        # Delete treatment.
        super(DeleteObjectModalView, self).delete(request, *args, **kwargs)

    def _get_object_name(self):
        """
        Get object name
        """
        object_name = ""
        if self.object:
            if self.field_for_name:
                object_name = getattr(
                    self.object, self.field_for_name, str(self.object)
                )
            else:
                object_name = str(self.object)

        return object_name

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(DeleteObjectModalView, self).get_context_data(**kwargs)
        # Add the object representation
        context["object_name"] = self._get_object_name()

        return context

    @staticmethod
    def get_modal_html_path():
        return "core_main_app/common/modals/delete_page_modal.html"

    @staticmethod
    def get_modal_js_path():
        return {"path": "core_main_app/common/js/modals/delete.js", "is_raw": False}


@method_decorator(login_required, name="dispatch")
class EditTemplateVersionManagerView(EditObjectModalView):
    form_class = EditTemplateForm
    document = TemplateVersionManager
    success_url = reverse_lazy("admin:core_main_app_templates")
    success_message = "Name edited with success."

    def _save(self, form):
        # Save treatment.
        try:
            template_version_manager_api.edit_title(
                self.object, form.cleaned_data.get("title"), request=self.request
            )
        except exceptions.NotUniqueError:
            form.add_error(
                None,
                "An object with the same name already exists. Please choose "
                "another name.",
            )
        except Exception as e:
            form.add_error(None, str(e))



""" REST views for the template API
"""
from django.http import Http404
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons.exceptions import XMLError
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template import api as template_api
from core_main_app.rest.template.serializers import TemplateSerializer
from core_main_app.utils.file import get_file_http_response
from core_main_app.utils.boolean import to_bool
from core_main_app.utils.xml import format_content_xml


class TemplateDetail(APIView):
    """Retrieve a Template."""

    def get_object(self, pk, request):
        """Get Template from db

        Args:

            pk: ObjectId
            request:

        Returns:

            Template
        """
        try:
            return template_api.get(pk, request=request)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Retrieve a Template

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Template
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            template_object = self.get_object(pk, request=request)

            # Serialize object
            serializer = TemplateSerializer(template_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {"message": "Template not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError:
            content = {"message": "Access Forbidden."}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class TemplateDownload(APIView):
    """Download a Template"""

    def get_object(self, pk, request):
        """Get Template from db

        Args:

            pk: ObjectId
            request:

        Returns:

            Template
        """
        try:
            return template_api.get(pk, request=request)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """Download the XSD file from a Template

        Args:

            request: HTTP request
            pk: ObjectId

        Examples:

            ../template/[template_id]/download
            ../template/[template_id]/download?pretty_print=false

        Returns:

            - code: 200
              content: XSD file
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            template_object = self.get_object(pk, request=request)

            # get xml content
            content = template_object.content

            # get format bool
            format = request.query_params.get("pretty_print", False)

            # format content
            if to_bool(format):
                content = format_content_xml(content)

            return get_file_http_response(
                content, template_object.filename, "text/xsd", "xsd"
            )
        except Http404:
            content = {"message": "Template not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except XMLError:
            content = {"message": "Content is not well formatted XML."}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except AccessControlError:
            content = {"message": "Access Forbidden."}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Integration Test for Template Rest API
"""

from rest_framework import status

from core_main_app.rest.template import views as template_rest_views
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)

from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.template.fixtures.fixtures import AccessControlTemplateFixture


class TestTemplateDownload(MongoIntegrationBaseTestCase):
    fixture = AccessControlTemplateFixture()

    def setUp(self):
        super(TestTemplateDownload, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            user,
            param={"pk": self.fixture.user1_template.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_with_param_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            user,
            param={"pk": self.fixture.user1_template.id},
            data={"pretty_print": "false"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_with_param_returns_formatted_content(self):
        # Arrange
        user = create_mock_user("1")
        expected_value = b'<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">\n  <xs:element name="tag"/>\n</xs:schema>\n'
        # Act
        response = RequestMock.do_request_get(
            template_rest_views.TemplateDownload.as_view(),
            user,
            param={"pk": self.fixture.user1_template.id},
            data={"pretty_print": "true"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.content, expected_value)



"""
    Serializers used throughout the Rest API
"""
from os.path import join

from django.http import Http404
from rest_framework.fields import CharField
from rest_framework.fields import FileField, SerializerMethodField
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_main_app.components.blob.api as blob_api
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.blob.models import Blob
from core_main_app.components.blob.utils import get_blob_download_uri

from django.urls import reverse
from core_main_app import settings
from urllib.parse import urljoin


class BlobSerializer(DocumentSerializer):
    """Blob serializer"""

    blob = FileField(write_only=True)
    handle = SerializerMethodField()
    upload_date = SerializerMethodField()
    if "core_linked_records_app" in settings.INSTALLED_APPS:
        pid = SerializerMethodField()

    class Meta(object):
        model = Blob
        fields = ["id", "user_id", "filename", "handle", "blob", "upload_date"]
        read_only_fields = ("id", "user_id", "filename", "handle", "upload_date")
        if "core_linked_records_app" in settings.INSTALLED_APPS:
            fields.append("pid")
            read_only_fields = read_only_fields + ("pid",)

    def get_pid(self, instance):
        """Return pid

        Args:
            instance:

        Returns:

        """
        # return pid  if assigned
        if "core_linked_records_app" not in settings.INSTALLED_APPS:
            return None
        else:
            from core_linked_records_app.components.blob import (
                api as linked_blob_api,
            )
            from core_linked_records_app.settings import ID_PROVIDER_SYSTEM_NAME

            try:
                sub_url = reverse(
                    "core_linked_records_provider_record",
                    kwargs={"provider": ID_PROVIDER_SYSTEM_NAME, "record": ""},
                )
                blob_pid = linked_blob_api.get_pid_for_blob(str(instance.id))

                return urljoin(settings.SERVER_URI, join(sub_url, blob_pid.record_name))
            except:
                return None

    def get_handle(self, instance):
        """Return handle

        Args:
            instance:

        Returns:

        """
        # get request from context
        request = self.context.get("request")
        # return download handle
        return get_blob_download_uri(instance, request)

    def get_upload_date(self, instance):
        """Return upload date

        Args:
            instance:

        Returns:

        """
        # Return instance generation time
        return str(instance.id.generation_time)

    def create(self, validated_data):
        """Create and return a new `Blob` instance, given the validated data.

        Args:
            validated_data:

        Returns:

        """
        # Create blob
        blob_object = Blob(
            filename=validated_data["blob"].name,
            user_id=str(self.context["request"].user.id),
        )
        # Set file content
        blob_object.blob = validated_data["blob"].file

        # Save the blob
        return blob_api.insert(blob_object, self.context["request"].user)


class DeleteBlobsSerializer(DocumentSerializer):
    """Delete Blob serializer."""

    id = CharField()

    class Meta(object):
        model = Blob
        fields = ("id",)

    def validate_id(self, id):
        """Validate id field

        Args:
            id:

        Returns:

        """
        request = self.context.get("request")
        try:
            blob_api.get_by_id(id, request.user)
        except DoesNotExist:
            raise Http404
        return id



""" Blob tags
"""
import re

from django import template

from core_main_app import settings
from core_main_app.utils.urls import get_blob_download_regex

register = template.Library()


@register.simple_tag(name="render_blob_links_in_span")
def render_blob_links_in_span(*args, **kwargs):
    """Find all blobs link then frame then with <span>

    Args:
        *args:
        **kwargs:

    Returns:

    """
    xml_string = kwargs["xml_string"]

    # Retrieve blob links using the default download URL
    url_blobs = get_blob_download_regex(xml_string)

    # Retrieve blob links using PID if the app is installed
    if "core_linked_records_app" in settings.INSTALLED_APPS:
        from core_linked_records_app.utils import blob as pid_blob_utils

        url_blobs += pid_blob_utils.get_blob_download_regex(xml_string)

    # Apply special template for the blob urls found
    blob_html_pattern = "<span class='blob-link' data-blob-url=\"{0}\">{0}</span>"

    for url in url_blobs:
        xml_string = xml_string.replace(url, blob_html_pattern.format(url))

    return xml_string



"""Query tool unit tests
"""
import copy
import re
from unittest import TestCase

from core_main_app.utils.query.mongo.prepare import (
    _compile_regex,
    _add_sub_document_root,
)


class TestCompileRegex(TestCase):
    def test_query_with_regex_returns_query_with_compiled_regex(self):
        # set query
        query = {"dot.notation": "/regex/"}
        # compile regex
        _compile_regex(query)
        # assert
        self.assertTrue(isinstance(query["dot.notation"], re.Pattern))

    def test_query_with_regex_on_multiple_levels_returns_query_with_compiled_regex(
        self,
    ):
        # set query
        query = {
            "$and": [
                {
                    "$or": [
                        {"dot.notation.1": "/regex/"},
                        {"dot.notation.1.#text": "/regex/"},
                    ]
                },
                {
                    "$or": [
                        {"dot.notation.2": {"$lt": 500}},
                        {"dot.notation.2": {"$lt": 500}},
                    ]
                },
            ]
        }
        # compile regex
        _compile_regex(query)
        # assert
        self.assertTrue(
            isinstance(query["$and"][0]["$or"][0]["dot.notation.1"], re.Pattern)
        )
        self.assertTrue(
            isinstance(query["$and"][0]["$or"][1]["dot.notation.1.#text"], re.Pattern)
        )
        self.assertFalse(
            isinstance(query["$and"][1]["$or"][0]["dot.notation.2"], re.Pattern)
        )
        self.assertFalse(
            isinstance(query["$and"][1]["$or"][1]["dot.notation.2"], re.Pattern)
        )

    def test_query_without_regex_returns_same_query(self):
        # set query
        query = {"dot.notation": {"$gt": 0}}
        # save original query
        query_origin = copy.deepcopy(query)
        # compile regex
        _compile_regex(query)
        # assert
        self.assertEquals(query, query_origin)

    def test_query_without_regex_on_multiple_levels_returns_same_query(self):
        # set query
        query = {
            "$or": [{"dot.notation": {"$gt": 0}}, {"dot.notation.#text": {"$gt": 0}}]
        }
        # save original query
        query_origin = copy.deepcopy(query)
        # compile regex
        _compile_regex(query)
        # assert
        self.assertEquals(query, query_origin)


class TestAddSubDocumentRoot(TestCase):
    def test_add_sub_document_root_to_query(self):
        # set query
        query = {"dot.notation": "test"}
        # set expected query
        expected_query = {"root.dot.notation": "test"}
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)

    def test_add_sub_document_root_to_and_query(self):
        # set query
        query = {
            "$and": [{"dot.notation": {"$gt": 0}}, {"dot.notation.#text": {"$gt": 0}}]
        }
        # set expected query
        expected_query = {
            "$and": [
                {"root.dot.notation": {"$gt": 0}},
                {"root.dot.notation.#text": {"$gt": 0}},
            ]
        }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)

    def test_add_sub_document_root_to_or_query(self):
        # set query
        query = {
            "$or": [{"dot.notation": {"$gt": 0}}, {"dot.notation.#text": {"$gt": 0}}]
        }
        # set expected query
        expected_query = {
            "$or": [
                {"root.dot.notation": {"$gt": 0}},
                {"root.dot.notation.#text": {"$gt": 0}},
            ]
        }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)

    def test_add_sub_document_root_to_and_or_query(self):
        # set query
        query = {
            "$and": [
                {
                    "$or": [
                        {"dot.notation.1": "test"},
                        {"dot.notation.1.#text": "/regex/"},
                    ]
                },
                {
                    "$or": [
                        {"dot.notation.2": {"$lt": 500}},
                        {"dot.notation.2": {"$lt": 500}},
                    ]
                },
            ]
        }
        # set expected query
        expected_query = {
            "$and": [
                {
                    "$or": [
                        {"root.dot.notation.1": "test"},
                        {"root.dot.notation.1.#text": "/regex/"},
                    ]
                },
                {
                    "$or": [
                        {"root.dot.notation.2": {"$lt": 500}},
                        {"root.dot.notation.2": {"$lt": 500}},
                    ]
                },
            ]
        }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)



from os import chdir, pardir
from os.path import join, exists, dirname, normpath, abspath
from re import sub

from setuptools import find_packages, setup


def req_link(external_url):
    egg_link = sub(r"https://[^=]+=", "", external_url)
    return "==".join(egg_link.rsplit("-", 1))


def read_requirements_file(requirements_filepath):
    requirements = list()

    if not exists(requirements_filepath):
        return requirements

    with open(requirements_filepath) as requirements_fp:
        requirements += requirements_fp.read().splitlines()

    return requirements


reqs_dev = join(dirname(__file__), "requirements.dev.txt")
reqs_default = join(dirname(__file__), "requirements.txt")
reqs_core = join(dirname(__file__), "requirements.core.txt")
required = []

required += read_requirements_file(reqs_default)
required += read_requirements_file(reqs_core)

extra = read_requirements_file(reqs_dev)

dep_links = [r for r in required if r.startswith("https://")]
required = [req_link(r) if r.startswith("https://") else r for r in required]

with open(join(dirname(__file__), "README.rst")) as f:
    long_desc = f.read()

# Allow setup.py to be run from any path
chdir(normpath(join(abspath(__file__), pardir)))

setup(
    name="core_main_app",
    version="1.21.6",
    description="Main functionalities for the curator core project",
    long_description=long_desc,
    author="NIST IT Lab",
    author_email="itl_inquiries@nist.gov",
    url="https://github.com/usnistgov/core_main_app",
    packages=find_packages(),
    include_package_data=True,
    install_requires=required,
    dependency_links=dep_links,
    extras_require={"develop": extra},
)



""" Mock a Request object.
"""
import json
from unittest.mock import Mock

from django.contrib.sessions.models import Session
from django.core.wsgi import get_wsgi_application
from django.http import HttpResponse, HttpRequest
from rest_framework import status
from rest_framework.test import APIRequestFactory


class RequestMock(object):
    """Represent a request.
    Use this class to simulate an HTTP request.
    """

    @staticmethod
    def do_request_get(view, user, data=None, param=None):
        """Execute a GET HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("GET", view, user, data, param)

    @staticmethod
    def do_request_post(
        view, user, data=None, param=None, content_type="application/json"
    ):
        """Execute a POST HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.

        Returns:
            Response: Request response.
        """
        return RequestMock._do_request("POST", view, user, data, param, content_type)

    @staticmethod
    def do_request_put(
        view, user, data=None, param=None, content_type="application/json"
    ):
        """Execute a PUT HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.
            content_type: Content-Type of the data passed.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("PUT", view, user, data, param, content_type)

    @staticmethod
    def do_request_delete(
        view, user, data=None, param=None, content_type="application/json"
    ):
        """Execute a DELETE HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.
            content_type: Content-Type of the data passed.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("DELETE", view, user, data, param, content_type)

    @staticmethod
    def do_request_patch(
        view, user, data=None, param=None, content_type="application/json"
    ):
        """Execute a PATCH HTTP request.
        Args:
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.
            content_type: Content-Type of the data passed.

        Returns:
            Response: Request response.

        """
        return RequestMock._do_request("PATCH", view, user, data, param, content_type)

    @staticmethod
    def _do_request(
        http_method, view, user, data=None, param=None, content_type="application/json"
    ):
        """Execute the http_method request.
        Args:
            http_method: HTTP method.
            view: View method called by the request.
            user: User for the request.
            data: Data.
            param: View method params.
            content_type: Content-Type of the data passed.

        Returns:
            Response: Request response.

        """
        # Pre-process data depending on its content-type. GET request don't have
        # a content-type and the processing is bypassed in this case.
        if http_method != "GET":
            if content_type == "application/json":
                data = json.dumps(data)

        url = "/dummy_url"
        factory = APIRequestFactory()
        # Request by http_method.
        if http_method == "GET":
            request = factory.get(url, data=data)
        elif http_method == "POST":
            request = factory.post(url, data=data, content_type=content_type)
        elif http_method == "PUT":
            request = factory.put(url, data=data, content_type=content_type)
        elif http_method == "DELETE":
            request = factory.delete(url, data=data, content_type=content_type)
        elif http_method == "PATCH":
            request = factory.patch(url, data=data, content_type=content_type)
        else:
            return HttpResponse(status=status.HTTP_405_METHOD_NOT_ALLOWED)
        # Set the user
        request.user = user
        # Set the session
        request.session = Session(session_key="KEY")
        # i18n. Get django validation messages.
        get_wsgi_application()
        # Do not use CSRF checks.
        request._dont_enforce_csrf_checks = True

        if param:
            view_ = view(request, **param)
        else:
            view_ = view(request)

        return view_


def create_mock_request(user=None):
    """Create a Mock HTTP Request

    Args:
        user:

    Returns:

    """
    mock_request = Mock(spec=HttpRequest)
    mock_request.user = user
    mock_request.session = Session(session_key="KEY")
    return mock_request



"""
Template models
"""
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions
from core_main_app.utils.validation.regex_validation import not_empty_or_whitespaces
from django_mongoengine import fields, Document


class Template(Document):
    """Represents an XML schema template that defines the structure of data"""

    filename = fields.StringField(validation=not_empty_or_whitespaces)
    content = fields.StringField()
    user = fields.StringField(blank=True)
    hash = fields.StringField()
    _display_name = fields.StringField(blank=True)
    dependencies = fields.ListField(
        fields.ReferenceField("self"), default=[], blank=True
    )

    meta = {"allow_inheritance": True}

    @staticmethod
    def get_all(is_cls, users=None):
        """Return all templates.

        Args:
            is_cls (bool): True if filtering by templates only.
            users (list|None): List of users having access to the template.

        Returns:
            list<Template> - List of template following the query parameters.
        """
        template_query_kwargs = dict()

        if is_cls:  # will return all Template object only
            template_query_kwargs["_cls"] = Template.__name__

        if users is not None:  # select specific user if it is defined
            template_query_kwargs["user__in"] = users

        return Template.objects(**template_query_kwargs).all()

    @staticmethod
    def get_by_id(template_id):
        """Return a template by its id.

        Args:
            template_id:

        Returns:

        """
        try:
            return Template.objects().get(pk=str(template_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as e:
            raise exceptions.ModelError(str(e))

    @staticmethod
    def get_all_by_hash(template_hash, users):
        """Return all template having the given hash.

        Args:
            template_hash: Template hash.
            users:

        Returns:
            List of Template instance.

        """
        if users is not None:
            return Template.objects(hash=template_hash, user__in=users).all()
        return Template.objects(hash=template_hash).all()

    @staticmethod
    def get_all_by_hash_list(template_hash_list, users):
        """Return all template having the given hash list.

        Args:
            template_hash_list: Template hash list.
            users:

        Returns:
            List of Template instance.

        """
        if users is not None:
            return Template.objects(hash__in=template_hash_list, user__in=users).all()
        return Template.objects(hash__in=template_hash_list).all()

    @staticmethod
    def get_all_by_id_list(template_id_list, users=None):
        """Return all template with id in list.

        Args:
            template_id_list:
            users:

        Returns:

        """
        if users is not None:
            return Template.objects(pk__in=template_id_list, user__in=users).all()
        return Template.objects(pk__in=template_id_list).all()

    @property
    def display_name(self):
        """Return template name to display.

        Returns:

        """
        if self._display_name is not None:
            return self._display_name
        else:
            return self.filename

    @display_name.setter
    def display_name(self, value):
        """Set template name to display.

        Args:
            value:

        Returns:

        """
        self._display_name = value
